<?php

/**
 * @file
 * Builds placeholder replacement tokens for cms_content_sync-related data.
 */
use Drupal\Core\Render\BubbleableMetadata;
use Drupal\cms_content_sync\Entity\EntityStatus;

/**
 * Implements hook_token_info()
 */
function cms_content_sync_token_info()
{
  $info = [];

  $info['types']['cms_content_sync'] = [
    'name' => t('CMS Content Sync'),
    'description' => t('CMS Content Sync related tokens')
  ];

  $info['tokens']['cms_content_sync']['source_url'] = [
    'name' => t('Source URL'),
    'description' => t('The entities source URL. Does only work for Nodes. Mostly used for the canonical URL.')
  ];

  return $info;
}

/**
 * Implements hook_tokens()
 */
function cms_content_sync_tokens($type, $tokens, array $data, array $options, BubbleableMetadata $bubbleable_metadata)
{
  $replacements = [];
  if ($type == 'cms_content_sync') {

    foreach ($tokens as $name => $original) {
      // Find the desired token by name.
      switch ($name) {
        case 'source_url':
          $replacements[$original] = _cms_content_sync_tokens_source_url($data);
          break;
      }
    }
  }

  return $replacements;
}

/**
 * Token callback for the source_url.
 */
function _cms_content_sync_tokens_source_url($data)
{
  // Only support node entities.
  if (!empty($data['node'])) {
    /** @var \Drupal\node\NodeInterface $node */
    $node = $data['node'];

    // Copied from node.tokens.inc
    $url_options = ['absolute' => TRUE];
    if (isset($options['langcode'])) {
      $url_options['language'] = \Drupal::languageManager()->getLanguage($options['langcode']);
    }

    // Return the node_url if no status_entity or no source_url is set.
    $node_url = $node->toUrl('canonical', $url_options)->toString();

    $status_entities = EntityStatus::getInfosForEntity('node', $node->uuid());
    if (!empty($status_entities)) {
      // We only handle the first status entity.
      $status_entity = reset($status_entities);
      $source_url = $status_entity->getSourceUrl();

      if (isset($source_url)) {
        return $source_url;
      } else {
        return $node_url;
      }
    } else {
      return $node_url;
    }
  }

  return '';
}
